@@ -52,9 +52,11 @@ from .utils import (
                 get_file_meta_information,
                 parse_string_header,
                 format_string_for_id,
+                silent_drop_kwarg,
                 current_datetime,
                 file_and_fileobj,
                 detect_mimetype,
+                dict_to_query,
                 clean_kwargs,
                 current_date,
                 json_load,
@@ -66,32 +68,20 @@ from .config import Config
 try:
     from motor.motor_asyncio import AsyncIOMotorClient as MongoClient
     from motor.motor_asyncio import AsyncIOMotorGridFSBucket as GridFSBucket
-    SUPPORT_ASYNCIO_CLIENT = True
-    SUPPORT_ASYNCIO_BUCKET = True
+    SUPPORT_ASYNCIO_CLIENT = True #: True if motor package is installed else False
+    SUPPORT_ASYNCIO_BUCKET = True #: True if motor package is installed else False
 except:
     logger.warning("motor is not installed. needed if using asyncio")
     class MongoClient: pass
-    class GridFSBucket: pass
-    SUPPORT_ASYNCIO_CLIENT = False
-    SUPPORT_ASYNCIO_BUCKET = False
-
-def get_async_client():
-    global SUPPORT_ASYNCIO_CLIENT
-    if not SUPPORT_ASYNCIO_CLIENT:
-        raise Exception("motor not installed")
-    return AsyncIOClient
-
-def get_async_doc():
-    global SUPPORT_ASYNCIO_CLIENT
-    if not SUPPORT_ASYNCIO_CLIENT:
-        raise Exception("motor not installed")
-    return AsyncIODoc
+    class GridFSBucket: pass # NOTE: in case of refereneces
+    SUPPORT_ASYNCIO_CLIENT = False #: True if motor package is installed else False
+    SUPPORT_ASYNCIO_BUCKET = False #: True if motor package is installed else False
 
 
 class AsyncIOClient(MongoClient):
     """
 High-level AsyncIOMotorClient subclass with additional methods added for ease-of-use,
-having some automated conveniences and default argument values.
+having some automated conveniences and defaults.
     """
     _MONGO_URI = lambda _: getattr(Config, "MONGO_URI", None)
     _DEFAULT_COLLECTION = None
@@ -131,21 +121,25 @@ having some automated conveniences and default argument values.
                 self.FILES = GridFSBucket(db)
             else:
                 logger.warning("gridfsbucket not instantiated due to missing 'tornado' package")
-                self.FILES = GridFSBucket() # Using empty object
+                self.FILES = None
 
     def __repr__(self):
-        return "<cervmongo.AsyncIOClient>"
+        db = self.get_default_database()
+
+        if not getattr(db, "name", None) or db.name == "None":
+            return "<cervmongo.AsyncIOClient>"
+        else:
+            return f"<cervmongo.AsyncIOClient.{db.name}>"
 
     def _process_record_id_type(self, record):
         one = False
         if isinstance(record, str):
+            one = True
             if "$oid" in record:
                 record = json_load(record)
-                one = True
             else:
                 try:
-                    record = DOC_ID(record)
-                    one = True
+                    record = DOC_ID.__supertype__(record)
                 except:
                     pass
         elif isinstance(record, DOC_ID.__supertype__):
@@ -166,7 +160,10 @@ having some automated conveniences and default argument values.
             AsyncIOClient.__init__(self, mongo_uri=Config.MONGO_URI, default_collection=self._DEFAULT_COLLECTION)
 
     def COLLECTION(self, collection:str):
+
         self._DEFAULT_COLLECTION = collection
+
+
         class CollectionClient:
             __parent__ = CLIENT = self
             # INFO: variables
@@ -201,27 +198,28 @@ having some automated conveniences and default argument values.
                                 sort:PAGINATION_SORT_FIELDS=PAGINATION_SORT_FIELDS["_id"],
                                 after:str=None, before:str=None,
                                 page:int=None, endpoint:str="/",
-                                query:dict={}, **kwargs):
-        """Returns paginated results of collection w/ query.
+                                ordering:int=-1, query:dict={}, **kwargs):
+        """
+            Returns paginated results of collection w/ query.
 
-Available pagination methods:
- - __Cursor-based (default)__
-    - after
-    - before
-    - limit (results per page, default 20)
- - __Time-based__ (a datetime field must be selected)
-    - sort (set to datetime field)
-    - after (records after this time)
-    - before (records before this time)
-    - limit (results per page, default 20)
- - __Offset-based__ (not recommended)
-    - limit (results per page, default 20)
-    - page
-"""
+            Available pagination methods:
+             - **Cursor-based (default)**
+                - after
+                - before
+                - limit (results per page, default 20)
+             - **Time-based** (a datetime field must be selected)
+                - sort (set to datetime field)
+                - after (records after this time)
+                - before (records before this time)
+                - limit (results per page, default 20)
+             - **Offset-based** (not recommended)
+                - limit (results per page, default 20)
+                - page
+        """
         collection = collection or self._DEFAULT_COLLECTION
         assert collection, "collection must be of type str"
 
-        if isinstance(sort, ENUM.__constraints__):
+        if isinstance(sort, ENUM.__supertype__):
             sort = sort.value
 
         total_docs = await self.GET(collection, query=query, count=True, empty=0)
@@ -233,12 +231,13 @@ Available pagination methods:
                 pagination_method = "time"
             cursor = await self.GET(collection, query=query,
                                     limit=limit, key=sort, before=before,
-                                    after=after, empty=[])
+                                    after=after, sort=ordering, empty=[])
         else:
+            assert page >= 1, "page must be equal to or greater than 1"
             pagination_method = "offset"
             cursor = await self.GET(collection, query=query,
-                                    perpage=limit, sort=sort, page=page,
-                                    empty=[])
+                                    perpage=limit, key=sort, page=page,
+                                    sort=ordering, empty=[])
 
         results = [ record async for record in cursor ]
 
@@ -268,11 +267,23 @@ Available pagination methods:
             if any((after, before)):
                 new_before = template.format(_id=_id, date=date)
 
+            if pagination_method in ("cursor", "time"):
+                if before:
+                    check_ahead = await self.GET(collection, query=query,
+                                            limit=limit, key=sort, before=new_before, empty=0, count=True)
+                    if not check_ahead:
+                        new_before = None
+                elif after:
+                    check_ahead = await self.GET(collection, query=query,
+                                            limit=limit, key=sort, after=new_after, empty=0, count=True)
+                    if not check_ahead:
+                        new_after = None
+
         response = {
             "data": results,
             "details": {
                 "pagination_method": pagination_method,
-                "query": json_dump(query),
+                "query": dict_to_query(query),
                 "sort": sort,
                 "unique_id": getattr(self, "_UNIQUE_ID", "_id"),
                 "total": total_docs,
@@ -327,11 +338,12 @@ Available pagination methods:
 
     def GENERATE_ID(self, _id=None):
         if _id:
-            return DOC_ID(_id)
+            return DOC_ID.__supertype__(_id)
         else:
-            return DOC_ID()
+            return DOC_ID.__supertype__()
 
     async def UPLOAD(self, fileobj, filename:str=None, content_type:str=None, extension:str=None, **kwargs):
+        assert self.FILES, "GridFS instance not initialized, run method 'set_database' with the desired database and try again"
         fileobj = file_and_fileobj(fileobj)
         metadata = get_file_meta_information(fileobj, filename=filename, content_type=content_type, extension=extension)
         filename = metadata['filename']
@@ -340,11 +352,13 @@ Available pagination methods:
         return file_id
 
     async def ERASE(self, filename_or_id, revision:int=-1):
+        assert self.FILES, "GridFS instance not initialized, run method 'set_database' with the desired database and try again"
         fs_doc = await self.DOWNLOAD(filename_or_id, revision=revision)
         await self.FILES.delete(fs_doc._id)
         await fs_doc.close()
 
     async def DOWNLOAD(self, filename_or_id=None, revision:int=-1, skip:int=None, limit:int=None, sort:int=-1, **query):
+        assert self.FILES, "GridFS instance not initialized, run method 'set_database' with the desired database and try again"
         revision = int(revision)
         if filename_or_id:
             if isinstance(filename_or_id, DOC_ID.__supertype__):
@@ -401,7 +415,7 @@ Available pagination methods:
             #print((_traceback()))
             pass
 
-    async def ADD_FIELD(self, collection, field:str, value='', data=False, query:dict={}):
+    async def ADD_FIELD(self, collection, field:str, value:typing.Union[typing.Dict, typing.List, str, int, float, bool]='', data=False, query:dict={}):
         if not collection:
             if hasattr(self, '_DEFAULT_COLLECTION'):
                 collection = self._DEFAULT_COLLECTION
@@ -423,7 +437,7 @@ Available pagination methods:
                 await self.PATCH(collection, record['_id'], {"$set": {
                     field: value}})
 
-    async def REMOVE_FIELD(self, collection, field:str, value='', query:dict={}):
+    async def REMOVE_FIELD(self, collection, field:str, query:dict={}) -> None:
         if not collection:
             collection = self._DEFAULT_COLLECTION
         assert collection, "collection must be of type str"
@@ -431,9 +445,9 @@ Available pagination methods:
         records = await self.GET(collection, query=query, distinct=True)
 
         for record in records:
-            await self.PATCH(collection, record, {"$unset": {field: value}})
+            await self.PATCH(collection, record, {"$unset": {field: ""}})
 
-    async def GET(self, collection, record=None, sort:int=1, key:str="_id", count:bool=None, search:str=None, fields:dict=None, page:int=None, perpage:int=False, limit:int=None, after:str=None, before:str=None, empty=None, distinct:str=None, one:bool=False, **kwargs):
+    async def GET(self, collection, id_or_query:typing.Union[DOC_ID, str, typing.Dict]={}, sort:int=1, key:str="_id", count:bool=None, search:str=None, fields:dict=None, page:int=None, perpage:int=False, limit:int=None, after:str=None, before:str=None, empty=None, distinct:str=None, one:bool=False, **kwargs):
         db = self.get_default_database()
         collection = collection or self._DEFAULT_COLLECTION
         assert collection, "collection not provided"
@@ -443,28 +457,26 @@ Available pagination methods:
         cols = list(set(collection))
         results = []
         number_of_results = len(cols)
-        query = kwargs.pop("query", {})
 
         if distinct == True:
             distinct = "_id"
 
-        if record:
-            record, _one = self._process_record_id_type(record)
-            one = _one if _one else one
-            if one:
-                query.update({"_id": record})
-            else:
-                query.update(record)
+        id_or_query, _one = self._process_record_id_type(id_or_query)
+        one = _one if _one else one
+        if _one:
+            query = {"_id": id_or_query}
+        else:
+            query = id_or_query
 
         for collection in cols:
             collection = db[collection]
 
-            if any([query, not record and not search]):
-                if count:
+            if query or not search:
+                if count and not limit:
                     if query:
                         results.append(await collection.count_documents(query, **kwargs))
                     else:
-                        results.append(collection.estimated_document_count(**kwargs))
+                        results.append(await collection.estimated_document_count(**kwargs))
                 elif distinct:
                     cursor = await collection.distinct(distinct, filter=query, **kwargs)
                     results.append(sorted(cursor))
@@ -473,13 +485,14 @@ Available pagination methods:
                     cursor = collection.find(query, fields, **kwargs)
                     results.append(cursor.sort([(key, sort)]).skip(total).limit(perpage))
                 elif limit:
-                    query = {"$and": [
-                                query
-                            ]}
+                    if any((query, after, before)):
+                        query = {"$and": [
+                                    query
+                                ]}
                     if after or before:
                         if after:
                             sort_value, _id_value = after.split("_")
-                            _id_value = DOC_ID(_id_value)
+                            _id_value = DOC_ID.__supertype__(_id_value)
                             query["$and"].append({"$or": [
                                             {key: {"$lt": _id_value}}
                                         ]})
@@ -488,7 +501,7 @@ Available pagination methods:
                                 query["$and"][-1]["$or"].append({key: {"$lt": sort_value}, "_id": {"$lt": _id_value}})
                         elif before:
                             sort_value, _id_value = before.split("_")
-                            _id_value = DOC_ID(_id_value)
+                            _id_value = DOC_ID.__supertype__(_id_value)
                             query["$and"].append({"$or": [
                                             {key: {"$gt": _id_value}}
                                         ]})
@@ -496,8 +509,15 @@ Available pagination methods:
                                 sort_value = dateparse(sort_value)
                                 query["$and"][-1]["$or"].append({key: {"$gt": sort_value}, "_id": {"$gt": _id_value}})
 
-                    cursor = collection.find(query, fields, **kwargs).sort([(key, -1)]).limit(limit)
-                    results.append(cursor)
+                    if count:
+                        try:
+                            cursor = await collection.count_documents(query, limit=limit, hint=[(key, sort)], **kwargs)
+                        except:
+                            cursor = len(await collection.find(query, fields, **kwargs).sort([(key, sort)]).to_list(limit))
+                        results.append(cursor)
+                    else:
+                        cursor = collection.find(query, fields, **kwargs).sort([(key, sort)]).limit(limit)
+                        results.append(cursor)
                 elif one:
                     val = await collection.find_one(query, fields, **kwargs)
                     results.append(val if val else empty)
@@ -545,23 +565,31 @@ Available pagination methods:
 
         return await self.GET(collection, search=search, **kwargs)
 
-    async def PUT(self, collection, record, many=False):
+    async def POST(self, collection, record_or_records:typing.Union[typing.List, typing.Dict]):
         db = self.get_default_database()
-        if not collection:
-            if hasattr(self, '_DEFAULT_COLLECTION'):
-                collection = self._DEFAULT_COLLECTION
+        collection = collection or self._DEFAULT_COLLECTION
         assert collection, "collection must be of type str"
-
         collection = db[collection]
 
-        if many:
-            return await collection.insert_many(record)
+        if isinstance(record_or_records, (list, tuple)):
+            return await collection.insert_many(record_or_records)
+        elif isinstance(record_or_records, dict):
+            return await collection.insert_one(record_or_records)
         else:
-            if not "_id" in record:
-                await self.logs.tool.insert_one({'name': 'db_put_noid',
-                                             'data': record,
-                                             'time': current_datetime()})
-            return await collection.insert_one(record)
+            raise TypeError("invalid record type '{}' provided".format(type(record_or_records)))
+
+    async def PUT(self, collection, record_or_records:typing.Union[typing.List, typing.Dict]):
+        db = self.get_default_database()
+        collection = collection or self._DEFAULT_COLLECTION
+        assert collection, "collection must be of type str"
+        collection = db[collection]
+
+        if isinstance(record_or_records, (list, tuple)):
+            return await collection.insert_many(record_or_records)
+        elif isinstance(record_or_records, dict):
+            return await collection.insert_one(record_or_records)
+        else:
+            raise TypeError("invalid record type '{}' provided".format(type(record_or_records)))
 
     async def REPLACE(self, collection, original, replacement:dict, upsert=False):
         db = self.get_default_database()
@@ -628,88 +656,113 @@ class AsyncIODoc(AsyncIOClient):
         standardized documents and adding json schema validation.
     """
     _MONGO_URI = lambda _: getattr(Config, "MONGO_URI", None)
-    _DEFAULT_COLLECTION:str = None
-    _UNIQUE_ID:str = None
-    _TEMPLATE_PATH:str = None
-    _SCHEMA_PATH:str = None
-    _MARSHMALLOW:str = False
-    _DEFAULT_VALUES:dict = None
-    _RESTRICTED_KEYS:list = None
+    _DOC_TYPE:str = None #: MongoDB collection to use
+    _DOC_ID:str = "_id"
+    _DOC_SAMPLE:str = None
+    _DOC_SCHEMA:str = None
+    _DOC_MARSHMALLOW:str = False
+    _DOC_DEFAULTS:dict = {}
+    _DOC_RESTRICTED_KEYS:list = []
+    _DOC_ENUMS:list = []
+    _DOC_SETTINGS:str = None
 
-    def __init__(self, _id=None, collection=None, mongo_uri=None, template_path=None, schema_path=None, unique_id=None, **kwargs):
+    def __init__(self, _id=None, doc_type:str=None, doc_sample:typing.Union[typing.Dict, str]=None, doc_schema:typing.Union[typing.Dict, str]=None, doc_id:str=None, mongo_uri:str=None, **kwargs):
         self._MONGO_URI = mongo_uri or self._MONGO_URI
         if callable(self._MONGO_URI):
             self._MONGO_URI = self._MONGO_URI()
         # INFO: set default collection
-        self._DEFAULT_COLLECTION = collection or self._DEFAULT_COLLECTION
-        assert self._DEFAULT_COLLECTION, "collection must be of type str"
+        self._DOC_TYPE = doc_type or self._DOC_TYPE
+        assert self._DOC_TYPE, "collection must be of type str"
+        self._DEFAULT_COLLECTION = self._DOC_TYPE
         # INFO: location for sample record, used as template
-        self._TEMPLATE_PATH = template_path or self._TEMPLATE_PATH
+        self._DOC_SAMPLE = doc_sample or self._DOC_SAMPLE
         # INFO: path to validation schema
-        self._SCHEMA_PATH = schema_path or self._SCHEMA_PATH
-        # INFO: sets the unique id field for the document, if any (cannot be _id)
-        self._UNIQUE_ID = unique_id or self._UNIQUE_ID
-        # INFO: sets an empty list if restricted_keys is None
-        self._RESTRICTED_KEYS = self._RESTRICTED_KEYS or []
-        # INFO: sets an empty dict if restricted_keys is None
-        self._DEFAULT_VALUES = self._DEFAULT_VALUES or {}
+        self._DOC_SCHEMA = doc_schema or self._DOC_SCHEMA
+        # INFO: sets the unique id field for the document type, if any (cannot be _id)
+        self._DOC_ID = doc_id or self._DOC_ID
+        assert self._DOC_ID, "unique id field name must be of type str"
 
         for kwarg in kwargs.keys():
-            if kwarg.lower() in ('marshmallow', 'default_values', 'restricted_keys'):
+            if kwarg.lower() in ('doc_settings', 'doc_marshmallow', 'doc_defaults', 'doc_restricted_keys'):
                 setattr(self, "_{}".format(kwarg.upper()), kwargs.pop(kwarg))
 
-        # Initial Record object with template else start blank dict
-        if self._TEMPLATE_PATH:
-            if isinstance(self._TEMPLATE_PATH, str):
-                template_full_path = os_path.join(Config.JSON_SAMPLE_PATH, self._TEMPLATE_PATH)
-                with open(template_full_path) as file1:
-                    template = json_load(file1.read())
-            elif isinstance(self._TEMPLATE_PATH, dict):
-                template = self._TEMPLATE_PATH
+        # Initial Record object with sample else start blank dict
+        if self._DOC_SAMPLE:
+            if isinstance(self._DOC_SAMPLE, str):
+                sample_full_path = os_path.join(Config.JSON_SAMPLE_PATH, self._DOC_SAMPLE)
+                with open(sample_full_path) as file1:
+                    sample = json_load(file1.read())
+            elif isinstance(self._DOC_SAMPLE, dict):
+                sample = self._DOC_SAMPLE
             else:
-                raise TypeError("_TEMPLATE_PATH is invalid type '{}', valid types are dict and str".format(type(self._TEMPLATE_PATH)))
-            parent_found = template.pop("__parent__", None)
-            while parent_found:
-                parent_full_path = os_path.join(Config.JSON_SAMPLE_PATH, parent_found)
-                with open(parent_full_path) as file1:
-                    parent = json_load(file1.read())
-                parent.update(template)
-                template = parent
-                parent_found = template.pop("__parent__", None)
-            self.template = template
+                raise TypeError("_DOC_SAMPLE is invalid type '{}', valid types are dict and str".format(type(self._DOC_SAMPLE)))
+            sample_parent_found = sample.pop("__parent__", None)
+            while sample_parent_found:
+                parent_sample_full_path = os_path.join(Config.JSON_SAMPLE_PATH, sample_parent_found)
+                with open(parent_sample_full_path) as _file:
+                    parent_sample = json_load(_file.read())
+                parent_sample.update(sample)
+                sample = parent_sample
+                sample_parent_found = sample.pop("__parent__", None)
+            self.sample = sample
         else:
-            self.template = {}
+            self.sample = {}
 
         # INFO: Load schema else start blank dict to add manual validation entries
-        if self._SCHEMA_PATH:
-            with open(os_path.join(Config.JSON__SCHEMA_PATH, self._SCHEMA_PATH)) as file1:
-                self.schema = json_load(file1.read())
+        if self._DOC_SCHEMA:
+            if isinstance(self._DOC_SCHEMA, str):
+                schema_full_path = os_path.join(Config.JSON_SCHEMA_PATH, self._DOC_SCHEMA)
+                with open(schema_full_path) as _file:
+                    self.schema = json_load(_file.read())
+            elif isinstance(self._DOC_SCHEMA, dict):
+                self.schema = self._DOC_SCHEMA
+            else:
+                raise TypeError("_DOC_SCHEMA is invalid type '{}', valid types are dict and str".format(type(self._DOC_SCHEMA)))
         else:
             self.schema = {}
 
         AsyncIOClient.__init__(self, **kwargs)
 
-        # INFO: If class has a _UNIQUE_ID assigned, create unique index
-        if self._UNIQUE_ID:
-            self.INDEX(self._DEFAULT_COLLECTION, key=self._UNIQUE_ID,
+        db = self.get_default_database()
+
+        if not getattr(db, "name", None) or db.name == "None":
+            raise Exception("database not provided in MongoDB URI")
+        else:
+            self._DOC_DB = db.name
+
+        # Initialize enums
+        if not self._DOC_ENUMS:
+            pass
+            #enums_record = self.GET(self._DOC_SETTINGS, "enums"
+
+        # INFO: If class has a _DOC_ID assigned, create unique index
+        if self._DOC_ID != "_id":
+            self.INDEX(self._DOC_TYPE, key=self._DOC_ID,
                                                         sort=1, unique=True)
 
         self.load(_id)
 
-    # ~ def __enter__(self):
-        # ~ return self
+    def __repr__(self):
+        if self.RECORD.get("_id", None):
+            _id = self.id()
+            return f"<cervmongo.AsyncIODoc.{self._DOC_DB}.{self._DOC_TYPE}.{self._DOC_ID}:{_id}>"
+        else:
+            return "<cervmongo.AsyncIODoc>"
 
-    # ~ def __exit__(self):
-        # ~ self.close()
+    def __enter__(self):
+        return self
+
+    def __exit__(self):
+        self.close()
 
     def _process_restrictions(self, record:dict=None):
         """removes restricted keys from record and return record"""
         try:
             if record:
                 assert isinstance(record, (MongoDictResponse, dict)), "Needs to be a dictionary, got {}".format(type(record))
-                return {key: value for key, value in record.items() if not key in self._RESTRICTED_KEYS}
+                return {key: value for key, value in record.items() if not key in self._DOC_RESTRICTED_KEYS}
             else:
-                return {key: value for key, value in self.RECORD.items() if not key in self._RESTRICTED_KEYS}
+                return {key: value for key, value in self.RECORD.items() if not key in self._DOC_RESTRICTED_KEYS}
         except:
             logger.exception("encountered error when cleaning self.RECORD, returning empty dict")
             return {}
@@ -776,12 +829,12 @@ class AsyncIODoc(AsyncIOClient):
     async def load(self, _id=None):
         # If _id specified on init, load actual record versus blank template
         if _id:
-            if self._UNIQUE_ID:
-                self.RECORD = await self.GET(self._DEFAULT_COLLECTION, query={self._UNIQUE_ID: _id}, one=True)
+            if self._DOC_ID:
+                self.RECORD = await self.GET(self._DOC_TYPE, query={self._DOC_ID: _id}, one=True)
             else:
-                self.RECORD = await self.GET(self._DEFAULT_COLLECTION, _id)
+                self.RECORD = await self.GET(self._DOC_TYPE, _id)
         else:
-            self.RECORD = copy.deepcopy(self.template)
+            self.RECORD = copy.deepcopy(self.sample)
 
         if not self.RECORD:
             self.RECORD = {}
@@ -790,71 +843,72 @@ class AsyncIODoc(AsyncIOClient):
                     data=self._p_r(self.RECORD),
                     details={
                 "state": "unsaved" if not self.RECORD.get("_id", None) else "saved",
-                "unique_id": self._UNIQUE_ID if self._UNIQUE_ID else "_id"
+                "unique_id": self._DOC_ID
                 })
 
     async def view(self, _id=False):
         if not _id:
             return StandardResponse(data=self._p_r(self.RECORD))
         else:
-            if self._UNIQUE_ID:
+            if self._DOC_ID:
                 return StandardResponse(
-                            data=self._p_r(self.GET(self._DEFAULT_COLLECTION, query={self._UNIQUE_ID: _id}, one=True, empty={})),
+                            data=self._p_r(self.GET(self._DOC_TYPE, query={self._DOC_ID: _id}, one=True, empty={})),
                             details={
-                        "unique_id": self._UNIQUE_ID if self._UNIQUE_ID else "_id"
+                        "unique_id": self._DOC_ID
                         }
                     )
             else:
                 return StandardResponse(
-                            data=self._p_r(self.GET(self._DEFAULT_COLLECTION, query={"_id": _id}, one=True, empty={})),
+                            data=self._p_r(self.GET(self._DOC_TYPE, query={"_id": _id}, one=True, empty={})),
                             details={
-                        "unique_id": self._UNIQUE_ID if self._UNIQUE_ID else "_id"
+                        "unique_id": self._DOC_ID
                         }
                     )
 
     async def reload(self):
         assert self.RECORD.get('_id')
-        self.RECORD = await self.GET(self._DEFAULT_COLLECTION, self.RECORD['_id'])
+        self.RECORD = await self.GET(self._DOC_TYPE, self.RECORD['_id'])
 
         return {
             "data": self._p_r(self.RECORD),
             "details": {
-                "unique_id": self._UNIQUE_ID if self._UNIQUE_ID else "_id"
+                "unique_id": self._DOC_ID
                 }
             }
 
     def id(self):
-        if self._UNIQUE_ID:
-            return self.RECORD.get(self._UNIQUE_ID, None)
-        else:
-            return self.RECORD.get("_id", None)
+        return self.RECORD.get(self._DOC_ID, None)
 
     async def create(self, save:bool=False, trigger=None, template:str="{total}", query:dict={}, **kwargs):
         assert self.RECORD.get('_id') is None, """Cannot use create method on
  an existing record. Use patch method instead."""
 
-        if self._MARSHMALLOW:
-            self._MARSHMALLOW().load(kwargs)
+        if self._DOC_MARSHMALLOW:
+            self._DOC_MARSHMALLOW().load(kwargs)
             self.RECORD.update(kwargs)
-        elif self.template:
-            assert all([ x in self.template for x in kwargs.keys()])
+        elif self.sample:
+            # INFO: removing invalid keys based on sample record
+            [ silent_drop_kwarg(kwargs, x, reason="not in self.sample") for x in list(kwargs.keys()) if not x in self.sample ]
             self.RECORD.update(kwargs)
         else:
             self.RECORD.update(kwargs)
 
-        kwargs['total'] = str(self.GET(self._DEFAULT_COLLECTION, query=query).count() + 1).zfill(6)
+        kwargs['total'] = str(self.GET(self._DOC_TYPE, query=query).count() + 1).zfill(6)
 
-        if self._UNIQUE_ID and not self.RECORD.get(self._UNIQUE_ID):
-            self.RECORD[self._UNIQUE_ID] = self._generate_unique_id(template=template, **kwargs)
+        if self._DOC_ID and not self.RECORD.get(self._DOC_ID):
+            self.RECORD[self._DOC_ID] = self._generate_unique_id(template=template, **kwargs)
 
         if save:
             await self.save(trigger=None)
 
         return {
             "data": self._p_r(self.RECORD),
-            "details": {"unique_id": self._UNIQUE_ID, "collection": self._DEFAULT_COLLECTION, "_id": self.RECORD[self._UNIQUE_ID]}
+            "details": {"unique_id": self._DOC_ID, "collection": self._DOC_TYPE, "_id": self.RECORD[self._DOC_ID]}
             }
 
+    async def add_enum(name:str, value):
+        self.RECORD
+
     async def push(self, **kwargs):
         assert self.RECORD.get('_id'), """Cannot use push method on
  a non-existing record. Use create method instead."""
@@ -961,11 +1015,11 @@ class AsyncIODoc(AsyncIOClient):
         if "_id" in kwargs:
             kwargs.pop("_id")
 
-        if self._MARSHMALLOW:
-            _MARSHMALLOW().load(kwargs, partial=True)
+        if self._DOC_MARSHMALLOW:
+            _DOC_MARSHMALLOW().load(kwargs, partial=True)
             self.RECORD.update(kwargs)
-        elif self.template:
-            assert all([ x in self.template for x in kwargs.keys()])
+        elif self.sample:
+            assert all([ x in self.sample for x in kwargs.keys()])
             self.RECORD.update(kwargs)
         else:
             self.RECORD.update(kwargs)
@@ -974,22 +1028,25 @@ class AsyncIODoc(AsyncIOClient):
 
         return {
             "data": self._p_r(self.RECORD),
-            "details": {}
+            "details": {
+                "processed": True,
+                "diff": kwargs
+                }
             }
 
     async def save(self, trigger=None):
         _id = None
 
-        if self._DEFAULT_VALUES:
-            for key, value in self._DEFAULT_VALUES.items():
+        if self._DOC_DEFAULTS:
+            for key, value in self._DOC_DEFAULTS.items():
                 if not self.RECORD.get(key):
                     self.RECORD[key] = value
 
         if self.RECORD.get('_id'):
             _id = self.RECORD.pop("_id")
         try:
-            if self._MARSHMALLOW:
-                self._MARSHMALLOW().load(self.RECORD)
+            if self._DOC_MARSHMALLOW:
+                self._DOC_MARSHMALLOW().load(self.RECORD)
             else:
                 validate(self.RECORD, self.schema)
         except:
@@ -1044,7 +1101,7 @@ class AsyncIODoc(AsyncIOClient):
             else:
                 self.RECORD[field] = {key: value if value else self.GENERATE_ID()}
         else:
-            template_path = os_path.join(_TEMPLATE_PATH, object_name + ".json")
+            template_path = os_path.join(_DOC_SAMPLE, object_name + ".json")
             assert os.path.exists(template_path), "path does not exist"
             with open(template_path) as file1:
                 self.RECORD[field] = json_util.loads(file1.read())
@@ -1055,3 +1112,18 @@ class AsyncIODoc(AsyncIOClient):
             "data": self._p_r(self.RECORD),
             "details": {field: self.RECORD[field], "state": "unsaved"}
             }
+
+
+def get_async_client() -> AsyncIOClient:
+    """returns AsyncIOClient class"""
+    global SUPPORT_ASYNCIO_CLIENT
+    if not SUPPORT_ASYNCIO_CLIENT:
+        raise Exception("motor not installed")
+    return AsyncIOClient
+
+def get_async_doc() -> AsyncIODoc:
+    """returns AsyncIODoc class"""
+    global SUPPORT_ASYNCIO_CLIENT
+    if not SUPPORT_ASYNCIO_CLIENT:
+        raise Exception("motor not installed")
+    return AsyncIODoc
